@@ -421,7 +421,7 @@ class MailSupport:
         """
         Has mailout been configured ?
         """
-        if (hasattr(self,'MailHost') and
+        if (self.mailhost() and
             (self.fromProperty() or self.replyToProperty())):
             return 1
         else:
@@ -467,7 +467,7 @@ class MailSupport:
         member's email property
         """
         address = (self.fromProperty() or
-                   self.usersEmailAddress() or
+                   #self.usersEmailAddress() or
                    self.replyToProperty())
         realname = self.usernameFrom(REQUEST,ip_address=0) or _('anonymous')
         return '%s (%s)' % (address, realname)
@@ -564,9 +564,22 @@ class MailSupport:
 
     def mailhost(self):
         """
-        Give the MailHost that should be used for sending mail.
+        Give the MailHost that should be used for sending mail, or None.
+
+        We want to find a mailhost, any mailhost, in a robust way without
+        confusion from pages named MailHost, and to acquire it from a
+        parent folder if necessary. So: we look for the first object of
+        Mail Host meta_type in this folder, then in the parent folder, and
+        so on. Overkill ? But it needs to just work.
         """
-        return self.MailHost
+        folder = self.folder()
+        mhost = None
+        # folder might not have objectValues, don't know why right now (#938)
+        while (not mhost) and folder and hasattr(folder,'objectValues'): # XXX
+            mhosts = folder.objectValues(spec='Mail Host')
+            if mhosts: mhost = mhosts[0]
+            folder = getattr(folder,'aq_parent',None)
+        return mhost
 
     def sendMailToSubscribers(self, text, REQUEST, subjectSuffix='',
                               subject='',message_id=None,in_reply_to=None,
@@ -579,10 +592,10 @@ class MailSupport:
         to prevent page edits, catch any mail-sending errors (and log and
         try to forward them to an admin).
 
-        In some cases we will skip some or all subscribers, eg if this
-        has been designated one of the "quiet" pages, but we do this
-        filtering down in sendMailTo. XXX that is, we would if we knew how.
+        As a special case, if text is empty we'll do nothing, to help
+        wikimail signal-to-noise ratio.
         """
+        if not text: return
         to = None
         recipients = self.subscriberList()
         if not self.title_or_id() in self.quietPages():
@@ -659,6 +672,7 @@ List-Subscribe: <%s/subscribeform>
 List-Unsubscribe: <%s/subscribeform>
 List-Archive: <%s>
 List-Help: <%s>
+Content-Type: text/plain; charset="utf-8"
 
 %s
 %s
@@ -684,7 +698,9 @@ List-Help: <%s>
         # send
         try:
             self.mailhost().send(msg)
-            BLATHER('sent mail:\n%s' % msg)
+            #BLATHER('sent mail:\n%s' % msg)
+            BLATHER('sent mail to subscribers:\nTo: %s\nBcc: %s' % (
+                tohdr,self.bccHeader(recipients)))
         # if there is any failure, notify admin or log
         except:
             BLATHER('failed to send mail to %s: %s' % (recipients,
@@ -692,7 +708,7 @@ List-Help: <%s>
             admin = getattr(self.folder(),'mail_admin',None)
             if admin:
                 try:
-                    self.sendMailTo(
+                    self.sendMailTo( #XXX possible infinite recursion ?
                         [],text,REQUEST,
                         subjectSuffix='ERROR, subscriber mailout failed',
                         to=admin)
