@@ -1,7 +1,7 @@
 """
 The main Zwiki module. See README.txt.
 
-(c) 1999-2003 Simon Michael <simon@joyful.com> for the zwiki community.
+(c) 1999-2004 Simon Michael <simon@joyful.com> for the zwiki community.
 Wikiwikiweb formatting by Tres Seaver <tseaver@zope.com>
 Parenting code and regulations by Ken Manheimer <klm@zope.com>
 Initial Zope CMF integration by Chris McDonough <chrism@zope.com>
@@ -64,42 +64,53 @@ from Regexps import url, bracketedexpr, doublebracketedexpr, \
      wikiname, wikilink, interwikilink, remotewikiurl, \
      protected_line, zwikiidcharsexpr, anywikilinkexpr, \
      markedwikilinkexpr, localwikilink, spaceandlowerexpr, \
-     dtmlorsgmlexpr
+     dtmlorsgmlexpr, wikinamewords, hashnumberexpr
 from Utils import Utils, BLATHER
 from UI import UI
 from OutlineSupport import OutlineSupport
-from Rating import RatingSupport
 from Diff import DiffSupport
 from Mail import SubscriberManagerMixin, MailSupport
 from CatalogAwareness import CatalogAwareness
-from Tracker import TrackerSupport
 from CMF import CMFAwareness
-from Fit import FitSupport
 from Comments import CommentsSupport
-from PurpleNumbers import PurpleNumbersSupport
 from Admin import AdminSupport
 from Editing import EditingSupport
 from I18nSupport import DTMLFile, _
-from pagetypes import MIDSECTIONMARKER, PAGETYPES, PAGE_TYPES, ZwikiStxPageType
+from pagetypes import PAGETYPES, PAGE_TYPES
+from pagetypes.common import MIDSECTIONMARKER
+from pagetypes.stx import ZwikiStxPageType
+from plugins import PLUGINS
 
 
 class ZWikiPage(
     EditingSupport, # use our ftp/dav methods, not DTMLDocument's
-    DTMLDocument,   # should DD really go last ?
+    DTMLDocument,   # should DD go last ?
     UI,
     OutlineSupport,
     DiffSupport,
     MailSupport,
     SubscriberManagerMixin,
     CatalogAwareness,
-    TrackerSupport,
     CMFAwareness,
-    FitSupport,
     CommentsSupport,
-    PurpleNumbersSupport,
-    RatingSupport,
     AdminSupport,
     Utils,
+    PLUGINS[0],     # runtime mixins hack, see plugins
+    PLUGINS[1],
+    PLUGINS[2],
+    PLUGINS[3],
+    PLUGINS[4],
+    PLUGINS[5],
+    PLUGINS[6],
+    PLUGINS[7],
+    PLUGINS[8],
+    PLUGINS[9],
+    PLUGINS[10],
+    PLUGINS[11],
+    PLUGINS[12],
+    PLUGINS[13],
+    PLUGINS[14],
+    PLUGINS[15],
     ):
     """
     A ZWikiPage is essentially a DTML Document which knows how to render
@@ -110,6 +121,7 @@ class ZWikiPage(
     """
     security = ClassSecurityInfo()
     security.declareObjectProtected('View')
+    security.declareProtected(Permissions.Edit, 'revert')
     security.declareProtected(Permissions.Edit, 'manage_upload')
     security.declareProtected(Permissions.FTP, 'manage_FTPstat') # needed
     security.declareProtected(Permissions.FTP, 'manage_FTPlist') # ?
@@ -213,35 +225,44 @@ class ZWikiPage(
         Render this zwiki page, also upgrading it on the fly if needed.
         """
         if AUTO_UPGRADE: self.upgrade(REQUEST)
-        # why do we need this ?
-        #if self.supportsCMF() and self.inCMF():
-        #    return CMFAwareness.__call__(self,client,REQUEST,RESPONSE,**kw)
-        #else:
-        if RESPONSE:
-            #XXX problem ? this is set by all templates
-            RESPONSE.setHeader('Content-Type', 'text/html') 
-            #RESPONSE.setHeader('Last-Modified', rfc1123_date(self._p_mtime)) 
-            #causes browser caching problems ? 
         rendered = self.render(client,REQUEST,RESPONSE,**kw)
         return rendered
 
     def render(self, client=None, REQUEST={}, RESPONSE=None, **kw):
         """
-        Render this zwiki page according to it's page_type.
+        Render this page according to it's page_type.
+
+        Also tries to ensure the HTTP content-type (and charset) have an
+        appropriate value. These may be set by the page type, or
+        overridden by a zwiki_content_type property (LatexWiki support).
+        NB this can also get set in I18nSupport.py.
         """
         if not self.preRendered(): self.preRender()
-        return self.pageType().render(self, REQUEST, RESPONSE, **kw)
+        r = self.pageType().render(self, REQUEST, RESPONSE, **kw)
+        if RESPONSE:
+            if hasattr(self,'zwiki_content_type'):
+                RESPONSE.setHeader('content-type',getattr(self,'zwiki_content_type'))
+            elif not RESPONSE.getHeader('content-type'):
+                RESPONSE.setHeader('content-type','text/html')
+        return r
 
     def preRender(self,clear_cache=0):
         """
         Make sure any applicable pre-rendering for this page has been done.
         
         If clear_cache is true, blow away any cached data.
+        XXX I think this happens anyway.
         """
         if clear_cache: self.clearCache()
         get_transaction().note('prerender')
         self.setPreRendered(self.pageType().preRender(self))
 
+    def renderText(self, text, type, **kw):
+        """
+        Render some source text according to the specified page type.
+        """
+        return self.lookupPageType(type)().renderText(self,text,**kw)
+            
     security.declareProtected(Permissions.View, 'clearCache')
     def clearCache(self,REQUEST=None):
         """
@@ -275,7 +296,7 @@ class ZWikiPage(
         if not self.hasDynamicContent(): return self.preRendered()
         return DTMLDocument.__call__(self,client,REQUEST,RESPONSE,**kw)
 
-    def renderMidsectionIn(self, text):
+    def renderMidsectionIn(self, text, **kw):
         """
         Insert some final things between the rendered document and discussion.
 
@@ -301,10 +322,10 @@ class ZWikiPage(
             discussion = '<a name="messages">' + discussion
         else:
             return text
-        return doc + self.renderMidsection() + discussion
+        return doc + self.renderMidsection(**kw) + discussion
 
-    def renderMidsection(self):
-        if self.subtopicsEnabled(): return self.subtopics(deep=1)
+    def renderMidsection(self,**kw):
+        if self.subtopicsEnabled(**kw): return self.subtopics(deep=1)
         else: return ''
     
     security.declareProtected(Permissions.View, 'supportsStx')
@@ -382,6 +403,7 @@ class ZWikiPage(
 
     def hasAllowedLinkSyntax(self,link):
         if (re.match(url,link) or
+            re.match(hashnumberexpr,link) or
             (self.wikinameLinksAllowed() and
              re.match(wikiname,link)) or
             (self.bracketLinksAllowed() and
@@ -439,7 +461,7 @@ class ZWikiPage(
 
     def renderLinksIn(self,text):
         """
-        Find and render all links in text.
+        Find and render all links in one step, without marking.
         """
         t = self.applyWikiLinkLineEscapesIn(text)
         t = re.sub(anywikilinkexpr,
@@ -463,9 +485,28 @@ class ZWikiPage(
         """
         return re.sub(wikilink, r'!\1', match.group(1))
 
+    security.declareProtected(Permissions.View, 'spacedWikinamesEnabled')
     def spacedWikinamesEnabled(self):
         """Should all wikinames be displayed with spaces in this wiki ?"""
-        return getattr(self.folder(),'spaced_wikinames',0) and 1
+        return getattr(self.folder(),'space_wikinames',0) and 1
+
+    security.declareProtected(Permissions.View, 'spacedNameFrom')
+    def spacedNameFrom(self,pagename):
+        """
+        Return pagename with spaces inserted if it's a WikiName, or unchanged.
+
+        Tries to be conformant with the wikiname regexp wrt. i18n, etc.
+        """
+        #spaced = pagename[0]
+        #for c in pagename[1:]:
+        #    if c in string.uppercase: spaced += ' '
+        #    spaced += c
+        #return spaced
+        if re.match('^%s$' % wikiname, pagename):
+            words = [x[0] for x in re.findall(wikinamewords,pagename)]
+            return ' '.join(words)
+        else:
+            return pagename
 
     security.declareProtected(Permissions.View, 'formatWikiname')
     def formatWikiname(self,wikiname):
@@ -473,16 +514,10 @@ class ZWikiPage(
         Convert a wikiname to this wiki's standard display format.
 
         Ie, leave it be or add ungodly spaces depending on the
-        'spaced_wikinames' property.
-
-        Not used yet, probably a waste of time.
+        'space_wikinames' property.
         """
         if self.spacedWikinamesEnabled():
-            spaced = wikiname[0]
-            for c in wikiname[1:]:
-                if c in string.uppercase: spaced += ' '
-                spaced += c
-            return spaced
+            return self.spacedNameFrom(wikiname)
         else: 
             return wikiname
 
@@ -533,8 +568,10 @@ class ZWikiPage(
                  self.doublebracketLinksAllowed()) or
                 (not re.match(doublebracketedexpr,link) and
                  self.bracketLinksAllowed())):
-                # convert to a page id if possible, and continue
-                p = self.pageWithFuzzyName(linknobrackets)
+                # yes - convert to the id of an existing page if possible,
+                # using fuzzy matching, and continue. 
+                p = self.pageWithFuzzyName(linknobrackets) or \
+                    self.issuePageWithNumber(self.issueNumberFrom(linknobrackets))
                 if p:
                     try: link = p.getId() # XXX poor caching
                     except: link = p.id # all-brains
@@ -543,6 +580,17 @@ class ZWikiPage(
         if re.match(url,link):
             return '<a href="%s">%s</a>' % (link, link)
 
+        # is it a hash number (#123) ?
+        if re.match(hashnumberexpr,link):
+            # yes - convert to the id of the issue page with that number
+            # and continue; if we can't, don't bother linking
+            p = self.issuePageWithNumber(self.issueNumberFrom(link))
+            if p:
+                try: link = p.getId() # XXX poor caching
+                except: link = p.id # all-brains
+            else:
+                return link
+
         # it must be a wikiname - are wikiname links allowed in this wiki ?
         if not (self.wikinameLinksAllowed() or
                 re.match(bracketedexpr,linkorig)): #was processed above
@@ -563,7 +611,7 @@ class ZWikiPage(
                    % self.pageWithNameOrId(link).issueColour # all-brains
             return '<a href="%s/%s" title="%s"%s%s>%s</a>' \
                    % (self.wiki_url(),quote(link),linktitle,accesskey,
-                      style,linknobrackets)
+                      style,self.formatWikiname(linknobrackets))
 
         # subwiki support: or does a matching page exist in the parent folder ?
         # XXX this is dumber than the above; doesn't handle i18n
@@ -572,12 +620,15 @@ class ZWikiPage(
               hasattr(self.folder().aq_parent, link) and
               self.isZwikiPage(getattr(self.folder().aq_parent,link))): #XXX poor caching
             return '<a href="%s/../%s" title="%s">../%s</a>'\
-                   % (self.wiki_url(),quote(link),_("page in parent wiki"),linkorig)
+                   % (self.wiki_url(),quote(link),_("page in parent wiki"),
+                      self.formatWikiname(linkorig))
 
         # otherwise, provide a creation link
         return ('%s<a class="new" href="%s/%s/createform?page=%s" title="%s">?</a>') \
-               % (linkorig, self.wiki_url(), quote(self.id()),
-                  quote(linknobrackets), _("create this page"))
+               % (self.formatWikiname(linkorig), self.wiki_url(),
+                  quote(self.id()), quote(linknobrackets),
+                  _("create this page"))
+                  
 
     def renderInterwikiLink(self, link):
         """
@@ -661,9 +712,12 @@ class ZWikiPage(
                 if lastlog: lastlog = ' ('+lastlog+')'
                 
                 # build the link around the interval
-                linked_interval = (' <a href="%(page_url)s/diff" title="' % self.page_url() +
-                                   _('show last edit %(lastlog)s') % {"lastlog":lastlog} +
-                                   ">" + interval + "</a>" )
+                linked_interval = (
+                    ' <a href="%s/diff" title="%s (%s)" >%s</a>' % (
+                    self.pageUrl(),
+                    _('show last edit'),
+                    lastlog, 
+                    interval))
                 
                 # use the link in a clear i18n way                                                                         
                 s =  _('last edited %(interval)s ago')  % {"interval": linked_interval}
@@ -704,6 +758,22 @@ class ZWikiPage(
         """
         return self.title_or_id()
     
+    security.declareProtected(Permissions.View, 'spacedPageName')
+    def spacedPageName(self):
+        """
+        Return this page's name, with spaces inserted if it's a WikiName.
+
+        We use this for eg the html title tag to improve search engine relevance.
+        """
+        return self.spacedNameFrom(self.pageName())
+    
+    security.declareProtected(Permissions.View, 'formattedPageName')
+    def formattedPageName(self):
+        """
+        Return this page's name in the standard display format (spaced or not).
+        """
+        return self.formatWikiname(self.pageName())
+
     security.declarePublic('Title')
     def Title(self):
         """
@@ -950,6 +1020,8 @@ class ZWikiPage(
         or the first page object in the folder, 
 	or None.
         """
+        # XXX need to handle a plone-like list property also
+        # XXX and will this pick up default_page from portal_properties ?
         return (
             self.pageWithName(getattr(self.folder(),'default_page','FrontPage'))
             or (list(self.pageObjects())+[None])[0]) # pageObjects may be a LazyMap
@@ -1033,8 +1105,9 @@ class ZWikiPage(
                 self.pageWithName(name,url_quoted))
         
     security.declareProtected(Permissions.View, 'pageWithFuzzyName')
-    def pageWithFuzzyName(self,name,url_quoted=0,
-                          allow_partial=0,ignore_case=1):
+    def pageWithFuzzyName(self,name,url_quoted=0,allow_partial=0,
+                          ignore_case=1, numeric_match=0):
+                          
         """
         Return the page in this folder for which name is a fuzzy link, or None.
 
@@ -1042,7 +1115,8 @@ class ZWikiPage(
         are multiple fuzzy matches, return the page whose name is
         alphabetically first.  The allow_partial flag allows even fuzzier
         matching. As of 0.17 ignore_case is not used and kept only for
-        backward compatibility.
+        backward compatibility. numeric_match modifies partial matching so
+        that [1] does not match a page "12...".
 
         performance-sensitive
         """
@@ -1063,7 +1137,9 @@ class ZWikiPage(
         for i in ids:
             ilower = string.lower(i)
             if (ilower == idlower or 
-                (allow_partial and ilower[:len(idlower)] == idlower)):
+                ((allow_partial and ilower[:len(idlower)] == idlower) and not
+                 (numeric_match and re.match(r'[0-9]',ilower[len(idlower):])))
+                ):
                 return self.pageWithId(i)
         return None
         
@@ -1144,6 +1220,7 @@ class ZWikiPage(
     zwiki_username_or_ip = Utils.usernameFrom
     applyLineEscapesIn = applyWikiLinkLineEscapesIn 
 
+
 InitializeClass(ZWikiPage)
 
 # rendering helpers
